<?php

namespace Drupal\entity_log;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Field\FieldItemList;
use Drupal\entity_log\Entity\EntityLog;

/**
 * Class EntityLogService.
 *
 * @package Drupal\entity_log
 */
class EntityLogService implements EntityLogServiceInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManager definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigFactory definition
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * EntityLogService constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   */
  public function __construct(EntityTypeManager $entity_type_manager, ConfigFactory $configFactory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $configFactory;
  }

  /**
   * Returns false or array of fields for logging
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return array|bool
   */
  public function entitySetForLogging(EntityInterface $entity){
    if($this->configFactory->get('entity_log.configuration')->get('log_in_logger') ||
      $this->configFactory->get('entity_log.configuration')->get('log_in_entity')){
      $config = $this->configFactory->get('entity_log.configuration')->get($entity->getEntityTypeId());
      return isset($config[$entity->bundle()]['fields']) ? array_filter($config[$entity->bundle()]['fields']) : false;
    }
    return false;
  }

  /**
   * Log fields into Entity Log entity
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param array $fields
   */
  public function logFields(EntityInterface $entity,array $fields){
    foreach ($fields as $field){
      if($entity->get($field) instanceof FieldItemList){
        $old_value = [];
        $new_value = [];
        /* @var \Drupal\options\Plugin\Field\FieldType\ListStringItem $item */
        foreach($entity->get($field)->getIterator() as $item){
          $new_value[] = $item->getString();
        }
        foreach($entity->original->get($field)->getIterator() as $item){
          $old_value[] = $item->getString();
        }
        if($new_value != $old_value){
          $old_value = implode(',',$old_value);
          $new_value = implode(',',$new_value);
          if($this->configFactory->get('entity_log.configuration')->get('log_in_logger')){
            \Drupal::logger('entity_log')->info(
              t('Entity type: @type | Bundle: @bundle | Field: @field_name | Old: @old | New: @new',[
                '@type'=>$entity->getEntityTypeId(),
                '@bundle'=>$entity->bundle(),
                '@field_name'=>$field,
                '@old'=>!empty($old_value) ? $old_value : 'empty',
                '@new'=>!empty($new_value) ? $new_value : 'empty'
              ]));
          }
          if($this->configFactory->get('entity_log.configuration')->get('log_in_entity')){
            EntityLog::create([
              "name"=>$field,
              "label"=>$entity->get($field)->getFieldDefinition()->getLabel(),
              "log_type" => $entity->getEntityTypeId(),
              'old_value'=>!empty($old_value) ? $old_value : 'empty',
              'new_value'=>!empty($new_value) ? $new_value : 'empty',
              "status" => 1,
              "entity_logged_id" => $entity,
            ])->save();
          }
        }
      }
    }
  }

}
